@@ -130,6 +130,7 @@ l7vs_lsock_get(struct sockaddr_in *sin, uint8_t proto, int backlog)
         struct l7vs_lsock *lsock;
         int stype;
         int ret;
+        int on = 1;
 
 	if(logger_get_log_level(LOG_CAT_L7VSD_SYSTEM_SOCKET) == LOG_LV_DEBUG){
                	LOGGER_PUT_LOG_DEBUG(LOG_CAT_L7VSD_SYSTEM_SOCKET,20,
@@ -213,6 +214,7 @@ l7vs_lsock_get(struct sockaddr_in *sin, uint8_t proto, int backlog)
                 return NULL;
         }
 
+        setsockopt(lsock->iom->fd, SOL_SOCKET, SO_REUSEADDR, (char*) &on, sizeof(int));
         ret = bind(lsock->iom->fd, (struct sockaddr *)sin, sizeof(*sin)); //binding the socket for incoming client request
         if (ret < 0) {
 		char addr_str[DEBUG_STR_LEN]={0};
