@@ -33,6 +33,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <glib.h>
+#include <sys/time.h>
 #include "logger_wrapper.h"
 #include "l7vs_config.h"
 #include "l7vs_service.h"
@@ -43,10 +44,8 @@
 #include "l7vs_lsock.h"
 #include "l7vs_sched.h"
 
-
 #define	SERVICE_REPLICATION_MAX_NUM	32
 
-
 //! service list pointer
 static GList * l7vs_service_list;
 
@@ -66,9 +65,11 @@ static  unsigned int    rep_size_num;
 static  l7vs_service_repdata * rep_mirror;
 
 //static functions
-static struct l7vs_service_arg_multi * l7vs_service_get_service_arg(struct l7vs_service *);
 static void l7vs_service_put_service_arg(struct l7vs_service_arg_multi *);
 
+// throughput interval
+static int throughput_interval = 0;
+
 /*!
  * make replication data from l7vs_service structure
  * @param[in]   num     unsigned int/number of array(l7vs_service_repdata)
@@ -118,8 +119,8 @@ make_replication_data( unsigned int num, struct l7vs_service_repdata* rep_data )
 		rep_data[i].sorry_addr  = svmulti->srv_arg.sorry_addr;
 		rep_data[i].sorry_flag  = svmulti->srv_arg.sorry_flag;
 		//QoS information
-		rep_data[i].qos_s       = svmulti->srv_arg.qos_s;
-		rep_data[i].qos_c       = svmulti->srv_arg.qos_c;
+		rep_data[i].qos_threshold_up   = svmulti->srv_arg.qos_threshold_up;
+		rep_data[i].qos_threshold_down = svmulti->srv_arg.qos_threshold_down;
 
 		l7vs_service_put_service_arg( svmulti );
 		//DEBUG output
@@ -195,8 +196,8 @@ set_replication_data( unsigned int num, struct l7vs_service_repdata* rep_data, l
 			memcpy( (void*)&srv->sorry_dest->addr, (void*)&rep_data[i].sorry_addr, sizeof(struct sockaddr_in) );
 			srv->sorry_dest->weight = 1;
 			srv->sorry_flag = rep_data[i].sorry_flag;
-			srv->QoS_threshold_service = rep_data[i].qos_s;
-			srv->QoS_threshold_conn    = rep_data[i].qos_c;
+			srv->qos_threshold_up   = rep_data[i].qos_threshold_up;
+			srv->qos_threshold_down = rep_data[i].qos_threshold_down;
 		}
 		break;
 	}
@@ -208,7 +209,7 @@ set_replication_data( unsigned int num, struct l7vs_service_repdata* rep_data, l
  * @param[in]	srv	l7vs_service pointer.
  * @return	l7vs_service_arg_multi pointer
  */
-static struct l7vs_service_arg_multi *
+struct l7vs_service_arg_multi *
 l7vs_service_get_service_arg(struct l7vs_service *srv)
 {
         struct l7vs_service_arg_multi *sa;
@@ -274,27 +275,40 @@ l7vs_service_get_service_arg(struct l7vs_service *srv)
 	sa->srv_arg.sorry_flag	= srv->sorry_flag;
 	memcpy( &sa->srv_arg.sorry_addr, &srv->sorry_dest->addr, sizeof(struct sockaddr_in) );
 
-	sa->srv_arg.qos_s	= srv->QoS_threshold_service;
-	sa->srv_arg.qos_c	= srv->QoS_threshold_conn;
+	sa->srv_arg.qos_threshold_up	= srv->qos_threshold_up;
+	sa->srv_arg.qos_threshold_down	= srv->qos_threshold_down;
 
-	if (srv->QoS_threshold_service != 0) {
-		srv->throughput_s >= srv->QoS_threshold_service
-				? sa->srv_arg.throughput_s = srv->QoS_threshold_service
-				: sa->srv_arg.throughput_s = srv->throughput_s;
+
+    struct timeval CurrTime;
+    gettimeofday( &CurrTime, NULL );
+    if (throughput_interval == 0) {
+        if ( parameter_is_int_exist( PARAM_COMP_L7VSD, "calc_throughput_interval" ) ) {
+            throughput_interval = parameter_get_int_value( PARAM_COMP_L7VSD, "calc_throughput_interval" );
+        }
+        if (throughput_interval == 0) {
+            throughput_interval = BPS_DEFAULT_INTERVAL;
+        }
+    }
+    unsigned long long cur_recvtime = (CurrTime.tv_sec * 1000000ULL + CurrTime.tv_usec) / throughput_interval;
+    l7vs_service_update_throughput(srv, cur_recvtime);
+    srv->throughput_to_server = srv->pre_recvsize_from_client * 1000000ULL / throughput_interval;
+    srv->throughput_to_client = srv->pre_recvsize_from_server * 1000000ULL / throughput_interval;
+
+
+	if (srv->qos_threshold_up != 0) {
+		srv->throughput_to_server >= srv->qos_threshold_up
+				? sa->srv_arg.throughput_to_server = srv->qos_threshold_up
+				: sa->srv_arg.throughput_to_server = srv->throughput_to_server;
 	} else {
-		sa->srv_arg.throughput_s = srv->throughput_s;
+		sa->srv_arg.throughput_to_server = srv->throughput_to_server;
 	}
 
-	if (srv->QoS_threshold_conn != 0) {
-		srv->throughput_c >= srv->QoS_threshold_conn
-				? sa->srv_arg.throughput_c = srv->QoS_threshold_conn
-				: sa->srv_arg.throughput_c = srv->throughput_c;
+	if (srv->qos_threshold_down != 0) {
+		srv->throughput_to_client >= srv->qos_threshold_down
+				? sa->srv_arg.throughput_to_client = srv->qos_threshold_down
+				: sa->srv_arg.throughput_to_client = srv->throughput_to_client;
 	} else {
-		sa->srv_arg.throughput_c = srv->throughput_c;
-	}
-
-	if (srv->QoS_threshold_service == 0 && srv->QoS_threshold_conn != 0) {
-		sa->srv_arg.throughput_s = sa->srv_arg.throughput_c;
+		sa->srv_arg.throughput_to_client = srv->throughput_to_client;
 	}
 
 	//DEBUG output
@@ -684,6 +698,7 @@ l7vs_service_create(struct l7vs_service_arg_multi *arg, int *err)
         }
 
 	// create new destination for sorry-server (weight not set)
+    // FIXME ignore IPv6 address???
 	sorry_dest = (struct l7vs_dest *)l7vs_dest_create((struct sockaddr_in *)&arg->srv_arg.sorry_addr, 0);
 	if (sorry_dest == NULL) {
 		l7vs_sched_put(sched);
@@ -793,12 +808,12 @@ l7vs_service_create(struct l7vs_service_arg_multi *arg, int *err)
         l7vs_sched_bind(sched, srv);
 
 	// set sorry data
-	srv->sorry_cc = arg->srv_arg.sorry_cc;
+	srv->sorry_cc   = arg->srv_arg.sorry_cc;
 	srv->sorry_dest = sorry_dest;
 	srv->sorry_flag = arg->srv_arg.sorry_flag;
 	// set QoS value
-	srv->QoS_threshold_service	= arg->srv_arg.qos_s;
-	srv->QoS_threshold_conn		= arg->srv_arg.qos_c;
+	srv->qos_threshold_up	= arg->srv_arg.qos_threshold_up;
+	srv->qos_threshold_down	= arg->srv_arg.qos_threshold_down;
 
         l7vs_lsock_add_service(lsock, srv);
         l7vs_service_list = g_list_append(l7vs_service_list, srv);
@@ -1426,13 +1441,13 @@ l7vs_service_flush_all(void)
 }
 
 /*!
- * set QoS(TraficControl) Threashold value
+ * set QoS(TraficControl) Threshold value
  * @param[in]	*srv	service pointer
  * @param[in]	*arg	l7vs_service_arg_multi pointer
  * return		0 = success / -1 = failur
  */
 int
-l7vs_service_set_QoS_Threashold( struct l7vs_service * srv, struct l7vs_service_arg_multi * arg )
+l7vs_service_set_QoS_Threshold( struct l7vs_service * srv, struct l7vs_service_arg_multi * arg )
 {
 //	struct l7vs_service_repdata *	service_replicationdata;
 
@@ -1449,7 +1464,7 @@ l7vs_service_set_QoS_Threashold( struct l7vs_service * srv, struct l7vs_service_
 	if( LOG_LV_DEBUG == logger_get_log_level( LOG_CAT_L7VSD_VIRTUAL_SERVICE ) ){
 		char	debugstr[DEBUG_STR_LEN];
 		memset( debugstr, 0, DEBUG_STR_LEN );
-		sprintf( debugstr, "function l7vs_service_set_QoS_Threashold( struct l7vs_service * srv, struct l7vs_service_arg_multi * arg )" );
+		sprintf( debugstr, "function l7vs_service_set_QoS_Threshold( struct l7vs_service * srv, struct l7vs_service_arg_multi * arg )" );
 		LOGGER_PUT_LOG_DEBUG( LOG_CAT_L7VSD_VIRTUAL_SERVICE,79, debugstr );
 		memset( debugstr, 0, DEBUG_STR_LEN );
 		sprintf( debugstr, "srv dump" );
@@ -1461,8 +1476,8 @@ l7vs_service_set_QoS_Threashold( struct l7vs_service * srv, struct l7vs_service_
 		LOGGER_PUT_LOG_DEBUG( LOG_CAT_L7VSD_VIRTUAL_SERVICE,81, debugstr );
 	}
 
-	srv->QoS_threshold_service	= arg->srv_arg.qos_s;
-	srv->QoS_threshold_conn		= arg->srv_arg.qos_c;
+	srv->qos_threshold_up	= arg->srv_arg.qos_threshold_up;
+	srv->qos_threshold_down	= arg->srv_arg.qos_threshold_down;
 
 	// Get the replication area.
 	unsigned int r_size = 0;
@@ -1474,7 +1489,7 @@ l7vs_service_set_QoS_Threashold( struct l7vs_service * srv, struct l7vs_service_
 	if( LOG_LV_DEBUG == logger_get_log_level( LOG_CAT_L7VSD_VIRTUAL_SERVICE ) ){
 		char	debugstr[DEBUG_STR_LEN];
 		memset( debugstr, 0, DEBUG_STR_LEN );
-		sprintf( debugstr, "function l7vs_service_set_QoS_Threashold( struct l7vs_service * srv, struct l7vs_service_arg_multi * arg ) return : 0" );
+		sprintf( debugstr, "function l7vs_service_set_QoS_Threshold( struct l7vs_service * srv, struct l7vs_service_arg_multi * arg ) return : 0" );
 		LOGGER_PUT_LOG_DEBUG( LOG_CAT_L7VSD_VIRTUAL_SERVICE,82, debugstr );
 	}
 
@@ -1570,7 +1585,7 @@ l7vs_service_get_VSnum()
  * return	l7vs_service pointer
  */
 struct l7vs_service *
-l7vs_servicw_get_VSInfo_byNum( unsigned int num )
+l7vs_service_get_VSInfo_byNum( unsigned int num )
 {
 	unsigned int vs_num = g_list_length( l7vs_service_list );
 	if( 0 > num )return NULL;
@@ -1579,11 +1594,11 @@ l7vs_servicw_get_VSInfo_byNum( unsigned int num )
 	if( LOG_LV_DEBUG == logger_get_log_level( LOG_CAT_L7VSD_VIRTUAL_SERVICE ) ){
 		char	debugstr[DEBUG_STR_LEN];
 		memset( debugstr, 0, DEBUG_STR_LEN );
-		sprintf( debugstr, "function l7vs_servicw_get_VSInfo_byNum( unsigned int num )" );
+		sprintf( debugstr, "function l7vs_service_get_VSInfo_byNum( unsigned int num )" );
 		sprintf( debugstr, "%s num = %d", debugstr, num );
 		LOGGER_PUT_LOG_DEBUG( LOG_CAT_L7VSD_VIRTUAL_SERVICE,89, debugstr );
 		memset( debugstr, 0, DEBUG_STR_LEN );
-		sprintf( debugstr, "function l7vs_servicw_get_VSInfo_byNum( unsigned int num ) return" );
+		sprintf( debugstr, "function l7vs_service_get_VSInfo_byNum( unsigned int num ) return" );
 		LOGGER_PUT_LOG_DEBUG( LOG_CAT_L7VSD_VIRTUAL_SERVICE,90, debugstr );
 	}
 	return (struct l7vs_service *)g_list_nth_data( l7vs_service_list, num );
@@ -1628,3 +1643,35 @@ l7vs_service_get_VSInfo_byHandler( handle_t in_handle )
 	return	NULL;
 }
 
+void
+l7vs_service_update_throughput( struct l7vs_service* srv, unsigned long long current_time ) {
+	if ( LOG_LV_DEBUG == logger_get_log_level(LOG_CAT_L7VSD_NETWORK_BANDWIDTH) ) {
+		char	srv_str[DEBUG_STR_LEN];
+		memset( srv_str, 0, DEBUG_STR_LEN );
+        l7vs_service_c_str(srv_str, srv);
+		LOGGER_PUT_LOG_DEBUG(LOG_CAT_L7VSD_NETWORK_BANDWIDTH,1,"in_function: void l7vs_service_update_throughput(struct l7vs_service* srv, unsigned long long current_time) srv=%s, current_time=%llu", srv_str, current_time);
+	}
+    if (srv->pre_recvtime_from_client == current_time - 2) {
+        srv->pre_recvtime_from_client = current_time - 1;
+        srv->pre_recvsize_from_client = srv->cur_recvsize_from_client;
+        srv->cur_recvsize_from_client = 0;
+    }
+    else if (srv->pre_recvtime_from_client != current_time - 1) {
+        srv->pre_recvtime_from_client = current_time - 1;
+        srv->pre_recvsize_from_client = 0;
+        srv->cur_recvsize_from_client = 0;
+    }
+    if (srv->pre_recvtime_from_server == current_time - 2) {
+        srv->pre_recvtime_from_server = current_time - 1;
+        srv->pre_recvsize_from_server = srv->cur_recvsize_from_server;
+        srv->cur_recvsize_from_server = 0;
+    }
+    else if (srv->pre_recvtime_from_server != current_time - 1) {
+        srv->pre_recvtime_from_server = current_time - 1;
+        srv->pre_recvsize_from_server = 0;
+        srv->cur_recvsize_from_server = 0;
+    }
+	if ( LOG_LV_DEBUG == logger_get_log_level(LOG_CAT_L7VSD_NETWORK_BANDWIDTH) ) {
+		LOGGER_PUT_LOG_DEBUG(LOG_CAT_L7VSD_VIRTUAL_SERVICE,2,"out_function: void l7vs_service_update_throughput(struct l7vs_service* srv, unsigned long long current_time)");
+	}
+}
